const {ccclass, property} = cc._decorator;

@ccclass
export default class ResCenter
{
    private static inst: ResCenter = null;
    private assetUrl: string = "http://10.26.172.161:8080/";

    public static get Me(): ResCenter
    {
        if(ResCenter.inst == null)
        {
            ResCenter.inst = new ResCenter();
        }

        return ResCenter.inst;
    }

    // version 是 hashVersion，在构建的时候需要勾选md5Cache选项，没有太仔细研究，应该不勾选就没有吧...
    public LoadBundle(bundleData: {bundleName: string, version: string}, isClear: boolean = false, successCallback: (bundle: cc.AssetManager.Bundle) => void): void
    {
        let localBundle: cc.AssetManager.Bundle = cc.assetManager.getBundle(bundleData.bundleName);
        
        // cc.assetManager.getBundle(); 可能在原生平台可以获取到，模拟器中每次都会清除缓存，所以获取不到
        // Tips: 如果已经下载过一个版本之后，就算版本号是不一样的也会重新加载之前的bundle，需要清除掉之前的bundle
        // cc.assetManager.removeBundle(); 清除掉之前的bundle，不知道需不需要清除缓存，这边是把缓存的文件也给清除掉了
        if(isClear == true && localBundle)
        {
            localBundle.releaseAll();
            this.RemoveBundle(bundleData.bundleName);
        }

        if(localBundle && isClear == false)
        {
            cc.log(localBundle);
            cc.log("缓存中存在 " + bundleData.bundleName + " 无需重新加载！version: " + bundleData.version);
            if(successCallback != null)
            {
                successCallback(localBundle);
            }
            return;
        }

        cc.log("缓存中不存在 " + bundleData.bundleName + " 进行重新加载！ version: " + bundleData.version);
        cc.assetManager.loadBundle(this.assetUrl + bundleData.bundleName, {version: bundleData.version}, (err, bundle) =>
        {
            if(err)
            {
                cc.error("bundle资源加载的失败！ bundleName: " + bundleData.bundleName);
                return;
            }

            if(successCallback != null)
            {
                successCallback(bundle);
            }
        })
    }

    public RemoveBundle(bundleName: string): void
    {
        let bundle: cc.AssetManager.Bundle = cc.assetManager.getBundle(bundleName);
        if(bundle != null)
        {
            cc.assetManager.removeBundle(bundle);
        }

        // 这里是清除缓存文件
        let assetCache: any = cc.assetManager.cacheManager.cachedFiles;
        assetCache.forEach((file, key) =>
        {
            if(file.bundle != undefined && file.bundle == bundle)
            {
                cc.log("清除的文件：" + key);
                cc.assetManager.cacheManager.removeCache(key);
            }
        })
    }

    // filePath 远程的路径，需要加上文件名以及后缀
    // Tips：类似这种 local/subGameInfo.txt
    public RemoveCacheFile(filePath: string): void
    {
        // 2.4.0 creator.d.ts removeCache 函数是没有携带参数的，需要改成 removeCache(originUrl: string): void;	
        // 不携带参数不会删除成功
        cc.assetManager.cacheManager.removeCache(this.assetUrl + filePath);
        cc.log("清除文件 ： " + this.assetUrl + filePath)
    }

    public BundleVersionIsIentical(bundleName: string, version: string): boolean
    {
        // 这里通过查找缓存中的json文件是否存在，存在就是版本一直，如果不携带hashVersion可能需要换一种方法
        let cache = cc.assetManager.cacheManager.getCache(this.assetUrl + bundleName + "/config." + version + ".json");
        cc.log(cc.assetManager.cacheManager.getCache(this.assetUrl + "SubGameInfo.txt"));
        cc.log(cache);
        if(cache)
        {
            return true;
        }

        return false;
    }

    public LoadResByRemote(resPath: string, successCallback: (r: any) => void): void
    {
        cc.assetManager.loadRemote(this.assetUrl + resPath, (err, res: any) =>
        {
            if(err)
            {
                cc.error("加载" + this.assetUrl + resPath +"资源失败！" + err);
                return;
            }

            if (successCallback != null) 
            {
                successCallback(res);
            }
        });
    }
}