import SubGameModel from '../Model/SubGameModel';
import { SubGameData } from '../Model/SubGameModel';
import ResCenter from '../Common/ResCenter';

const {ccclass, property} = cc._decorator;

@ccclass
export default class HallScene extends cc.Component 
{
    @property(cc.Node)
    SubGameGrid: cc.Node = null;

    @property(cc.Node)
    SubGameItem: cc.Node = null;

    onLoad()
    {
        this.GetSubGameInfo();
        cc.log(cc.js._registeredClassNames);
    }

    public GetSubGameInfo(): void
    {
        // 我希望每次都获取是新的，所以我这边直接清除掉之前的文件重新获取了
        ResCenter.Me.RemoveCacheFile("SubGameInfo.txt");
        ResCenter.Me.LoadResByRemote("SubGameInfo.txt", (text: any) =>
        {
            cc.log(JSON.parse(text.text));
            let subGameInfo: any[] = JSON.parse(text.text);
            SubGameModel.FillSubGameDataList(subGameInfo);
            this.InitHallView();
        });
    }

    public InitHallView(): void
    {
        for(let key in SubGameModel.DataList)
        {
            let data: SubGameData = SubGameModel.DataList[key];
            let newItem: cc.Node = cc.instantiate(this.SubGameItem);
            newItem.active = true;
            newItem.parent = this.SubGameGrid;
            newItem.getChildByName("GameName").getComponent(cc.Label).string = data.SubGameName;
            newItem.name = data.SubGameName;
        }
    }

    public ClickSubGameCallback(event): void
    {
        let subGame: cc.Node = event.target;
        let subGameData: SubGameData = SubGameModel.DataList[subGame.name];
        let bundleData: {bundleName: string, version: string} = {bundleName: subGameData.SubGameAssetBundle, version: subGameData.SubGameVersion};
        let isClear: boolean = ResCenter.Me.BundleVersionIsIentical(subGameData.SubGameAssetBundle, subGameData.SubGameVersion) == false;
        cc.log("IsClear: " + isClear);
        ResCenter.Me.LoadBundle(bundleData, isClear, (bundle: cc.AssetManager.Bundle) =>
        {
            bundle.loadScene(subGameData.SubGameName + "StartScene", (err, res) =>
            {
                if(err)
                {
                    cc.error(err);
                    return;
                }
                cc.log("运行场景： " + subGameData.SubGameName + "StartScene");
                cc.director.runScene(res);
            });
        })
    }
}